@@ -26,6 +26,7 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/resource.h>
+#include <arpa/inet.h>
 
 #include "relay.h"
 #include "router.h"
@@ -41,6 +42,7 @@ enum conntype {
 typedef struct _connection {
 	int sock;
 	char takenby;   /* -2: being setup, -1: free, 0: not taken, >0: tid */
+	char srcaddr[24];  /* string representation of source address */
 	char buf[METRIC_BUFSIZ];
 	int buflen;
 	char needmore:1;
@@ -56,7 +58,11 @@ struct _dispatcher {
 	enum conntype type;
 	char id;
 	size_t metrics;
+	size_t blackholes;
 	size_t ticks;
+	size_t prevmetrics;
+	size_t prevblackholes;
+	size_t prevticks;
 	enum { RUNNING, SLEEPING } state;
 	char keep_running:1;
 	route *routes;
@@ -163,6 +169,8 @@ int
 dispatch_addconnection(int sock)
 {
 	size_t c;
+	struct sockaddr_in6 saddr;
+	socklen_t saddr_len = sizeof(saddr);
 
 	pthread_rwlock_rdlock(&connectionslock);
 	for (c = 0; c < connectionslen; c++)
@@ -202,6 +210,23 @@ dispatch_addconnection(int sock)
 		pthread_rwlock_unlock(&connectionslock);
 	}
 
+	/* figure out who's calling */
+	if (getpeername(sock, (struct sockaddr *)&saddr, &saddr_len) == 0) {
+		snprintf(connections[c].srcaddr, sizeof(connections[c].srcaddr),
+				"(unknown)");
+		switch (saddr.sin6_family) {
+			case PF_INET:
+				inet_ntop(saddr.sin6_family,
+						&((struct sockaddr_in *)&saddr)->sin_addr,
+						connections[c].srcaddr, sizeof(connections[c].srcaddr));
+				break;
+			case PF_INET6:
+				inet_ntop(saddr.sin6_family, &saddr.sin6_addr,
+						connections[c].srcaddr, sizeof(connections[c].srcaddr));
+				break;
+		}
+	}
+
 	(void) fcntl(sock, F_SETFL, O_NONBLOCK);
 	connections[c].sock = sock;
 	connections[c].buflen = 0;
@@ -225,7 +250,7 @@ int
 dispatch_addlistener_udp(int sock)
 {
 	int conn = dispatch_addconnection(sock);
-	
+
 	if (conn == -1)
 		return 1;
 
@@ -306,9 +331,9 @@ dispatch_connection(connection *conn, dispatcher *self)
 	/* try to read more data, if that succeeds, or we still have data
 	 * left in the buffer, try to process the buffer */
 	if (
-			(!conn->needmore && conn->buflen > 0) || 
+			(!conn->needmore && conn->buflen > 0) ||
 			(len = read(conn->sock,
-						conn->buf + conn->buflen, 
+						conn->buf + conn->buflen,
 						(sizeof(conn->buf) - 1) - conn->buflen)) > 0
 	   )
 	{
@@ -340,9 +365,10 @@ dispatch_connection(connection *conn, dispatcher *self)
 				*q = '\0';  /* can do this because we substract one from buf */
 
 				/* perform routing of this metric */
-				conn->destlen =
-					router_route(conn->dests, CONN_DESTS_SIZE,
-							conn->metric, firstspace, self->routes);
+				self->blackholes += router_route(
+						conn->dests, &conn->destlen, CONN_DESTS_SIZE,
+						conn->srcaddr,
+						conn->metric, firstspace, self->routes);
 
 				/* restart building new one from the start */
 				q = conn->metric;
@@ -458,7 +484,11 @@ dispatch_runner(void *arg)
 	int c;
 
 	self->metrics = 0;
+	self->blackholes = 0;
 	self->ticks = 0;
+	self->prevmetrics = 0;
+	self->prevblackholes = 0;
+	self->prevticks = 0;
 	self->state = SLEEPING;
 
 	if (self->type == LISTENER) {
@@ -636,6 +666,18 @@ dispatch_get_ticks(dispatcher *self)
 	return self->ticks;
 }
 
+/**
+ * Returns the wall-clock time consumed since last call to this
+ * function.
+ */
+inline size_t
+dispatch_get_ticks_sub(dispatcher *self)
+{
+	size_t d = self->ticks - self->prevticks;
+	self->prevticks += d;
+	return d;
+}
+
 /**
  * Returns the number of metrics dispatched since start.
  */
@@ -645,6 +687,40 @@ dispatch_get_metrics(dispatcher *self)
 	return self->metrics;
 }
 
+/**
+ * Returns the number of metrics dispatched since last call to this
+ * function.
+ */
+inline size_t
+dispatch_get_metrics_sub(dispatcher *self)
+{
+	size_t d = self->metrics - self->prevmetrics;
+	self->prevmetrics += d;
+	return d;
+}
+
+/**
+ * Returns the number of metrics that were explicitly or implicitly
+ * blackholed since start.
+ */
+inline size_t
+dispatch_get_blackholes(dispatcher *self)
+{
+	return self->blackholes;
+}
+
+/**
+ * Returns the number of metrics that were blackholed since last call to
+ * this function.
+ */
+inline size_t
+dispatch_get_blackholes_sub(dispatcher *self)
+{
+	size_t d = self->blackholes - self->prevblackholes;
+	self->prevblackholes += d;
+	return d;
+}
+
 /**
  * Returns whether this dispatcher is currently running, or not.  A
  * dispatcher is running when it is actively handling a connection, and
