@@ -226,10 +226,12 @@ do_usage(int exitcode)
 	printf("  -b  server send batch size, defaults to 2500\n");
 	printf("  -q  server queue size, defaults to 25000\n");
 	printf("  -S  statistics sending interval in seconds, defaults to 60\n");
+	printf("  -m  send statistics like carbon-cache.py, e.g. not cumulative\n");
 	printf("  -c  characters to allow next to [A-Za-z0-9], defaults to -_:#\n");
 	printf("  -d  debug mode: currently writes statistics to log, prints hash\n"
 	       "      ring contents and matching position in test mode (-t)\n");
-	printf("  -s  submission mode: write info about errors to log\n");
+	printf("  -s  submission mode: don't add any metrics to the stream like\n"
+	       "      statistics, report drop counts and queue pressure to log\n");
 	printf("  -t  config test mode: prints rule matches from input on stdin\n");
 	printf("  -H  hostname: override hostname (used in statistics)\n");
 
@@ -253,11 +255,12 @@ main(int argc, char * const argv[])
 	server **servers;
 	char *allowed_chars = NULL;
 	int i;
+	enum { SUB, CUM } smode = CUM;
 
 	if (gethostname(relay_hostname, sizeof(relay_hostname)) < 0)
 		snprintf(relay_hostname, sizeof(relay_hostname), "127.0.0.1");
 
-	while ((ch = getopt(argc, argv, ":hvdstf:i:l:p:w:b:q:S:c:H:")) != -1) {
+	while ((ch = getopt(argc, argv, ":hvdmstf:i:l:p:w:b:q:S:c:H:")) != -1) {
 		switch (ch) {
 			case 'v':
 				do_version();
@@ -269,6 +272,9 @@ main(int argc, char * const argv[])
 					mode = DEBUG;
 				}
 				break;
+			case 'm':
+				smode = SUB;
+				break;
 			case 's':
 				mode = SUBMISSION;
 				break;
@@ -514,7 +520,7 @@ main(int argc, char * const argv[])
 	}
 
 	logout("starting statistics collector\n");
-	collector_start(&workers[1], clusters, internal_submission);
+	collector_start(&workers[1], clusters, internal_submission, smode == CUM);
 
 	logout("startup sequence complete\n");
 
