@@ -38,7 +38,7 @@ int router_readconfig(cluster **clret, route **rret, const char *path, size_t qu
 void router_optimise(route **routes);
 size_t router_rewrite_metric(char (*newmetric)[METRIC_BUFSIZ], char **newfirstspace, const char *metric, const char *firstspace, const char *replacement, const size_t nmatch, const regmatch_t *pmatch);
 void router_printconfig(FILE *f, char mode, cluster *clusters, route *routes);
-size_t router_route(destination ret[], size_t retsize, char *metric, char *firstspace, route *routes);
+char router_route(destination ret[], size_t *retcnt, size_t retsize, char *srcaddr, char *metric, char *firstspace, route *routes);
 void router_test(char *metric_path, route *routes);
 server **router_getservers(cluster *clusters);
 void router_shutdown(void);
